/****************************************************************/
/*  File: kalman.cpp                                              */
/*  Sistemas de Vision Computacional Trabajo Final              */		
/*  Fichero basico para filtro de Kalman                     */
/****************************************************************/

#include <iostream>

#include <cv.h>
#include <highgui.h>

#include "kalman.hpp"

using namespace std;
using namespace cv;


KalmanFilter initKalman(const kfState8_t init)
{

    KalmanFilter kalf = KalmanFilter( STATE_SIZE, MEAS_SIZE, 0);

    // Transition matrix A has main 4th-upper diagonal set to identity   
    setIdentity(kalf.transitionMatrix, Scalar(1));
    kalf.transitionMatrix.diag(-4) = Scalar(1);

    //if you need something more complex, you can use
    //  float A[] = {1,0,0,0,1,0,0,0,
    // 		     0,1,0,0,0,1,0,0,
    // 		     0,0,1,0,0,0,1,0,
    // 		     0,0,0,1,0,0,0,1,
    // 		     0,0,0,0,1,0,0,0,
    // 		     0,0,0,0,0,1,0,0,
    // 		     0,0,0,0,0,0,1,0,
    // 		     0,0,0,0,0,0,0,1};
    // kalf.transitionMatrix = Mat(STATE_SIZE, STATE_SIZE, CV_32F, A);
    
    //Measurement matrix H is the identity
    setIdentity(kalf.measurementMatrix, Scalar(1));
    
    //Process noise covariance matrix is diagonal with values SIGMA_Q
    setIdentity(kalf.processNoiseCov, Scalar(SIGMA_Q));

    //Measurement noise covariance matrix is diagonal, with values SIGMA_R
    setIdentity(kalf.measurementNoiseCov, Scalar(SIGMA_R));

    //Error covariance matrix a posteriori
    setIdentity(kalf.errorCovPost, Scalar(SIGMA_P));

    //Init the filter state. use copyTo()! If you use '=', both
    //variables reference the same data in memory.
    Mat st = (Mat_<float>(8,1) << init.x, init.y, init.w, init.h, 
	      init.vx, init.vy, init.vw, init.vh);
	      
    st.copyTo(kalf.statePost);
    
   
    return kalf;

}

void updateKalman(KalmanFilter& kalf)
{
    
    Mat pred = kalf.predict();
 
    kalf.correct(pred);

    return;

}

void updateKalman(KalmanFilter& kalf, kfMeas4_t meas)
{
    Mat meas = kalf.predict();

    //update the values with our measurement

    meas.at<float>(0,0) = meas.x;
    meas.at<float>(1,0) = meas.y;
    meas.at<float>(2,0) = meas.w;
    meas.at<float>(3,0) = meas.h;

    kalf.correct(meas);

    return;
}




// vKalman * Init_Kalman(t_CvKalmanState8 init_state)
// {
//   float A[] = {1,0,0,0,1,0,0,0,
// 			   0,1,0,0,0,1,0,0,
// 			   0,0,1,0,0,0,1,0,
// 			   0,0,0,1,0,0,0,1,
// 			   0,0,0,0,1,0,0,0,
// 			   0,0,0,0,0,1,0,0,
// 			   0,0,0,0,0,0,1,0,
// 			   0,0,0,0,0,0,0,1};

//   float H[] = {1,0,0,0,0,0,0,0,
// 			   0,1,0,0,0,0,0,0,
// 			   0,0,1,0,0,0,0,0,
// 			   0,0,0,1,0,0,0,0};

//   float Q[] = {SIGMA_Q,0,0,0,0,0,0,0,
// 			   0,SIGMA_Q,0,0,0,0,0,0,
// 			   0,0,SIGMA_Q,0,0,0,0,0,
// 			   0,0,0,SIGMA_Q,0,0,0,0,
// 			   0,0,0,0,SIGMA_Q,0,0,0,
// 			   0,0,0,0,0,SIGMA_Q,0,0,
// 			   0,0,0,0,0,0,SIGMA_Q,0,
// 			   0,0,0,0,0,0,0,SIGMA_Q };

//   float P[] = {SIGMA_P,0,0,0,0,0,0,0,
// 			   0,SIGMA_P,0,0,0,0,0,0,
// 			   0,0,SIGMA_P,0,0,0,0,0,
// 			   0,0,0,SIGMA_P,0,0,0,0,
// 			   0,0,0,0,SIGMA_P,0,0,0,
// 			   0,0,0,0,0,SIGMA_P,0,0,
// 			   0,0,0,0,0,0,SIGMA_P,0,
// 			   0,0,0,0,0,0,0,SIGMA_P};

//   float R[] = {SIGMA_R,0,0,0,
// 			   0,SIGMA_R,0,0,
// 			   0,0,SIGMA_R,0,
// 			   0,0,0,SIGMA_R };

//   CvKalman *kalf = cvCreateKalman( STATE_SIZE, MEAS_SIZE, 0);

//   memcpy(kalf->measurement_matrix->data.fl, H, sizeof(H));
//   memcpy(kalf->transition_matrix->data.fl, A, sizeof (A));
//   memcpy(kalf->process_noise_cov->data.fl, Q, sizeof (Q));
//   memcpy(kalf->measurement_noise_cov->data.fl ,R, sizeof (R));
//   memcpy(kalf->error_cov_post->data.fl, P, sizeof (P));

//   cvmSet(kalf->state_post,0,0, init_state.x);
//   cvmSet(kalf->state_post,1,0, init_state.y);
//   cvmSet(kalf->state_post,2,0, init_state.w);
//   cvmSet(kalf->state_post,3,0, init_state.h);
//   cvmSet(kalf->state_post,4,0, init_state.vx);
//   cvmSet(kalf->state_post,5,0, init_state.vy);
//   cvmSet(kalf->state_post,6,0, init_state.vw);
//   cvmSet(kalf->state_post,7,0, init_state.vh);

//   return kalf;
// }

// void Update_Kalman( CvKalman *kalf, t_CvKalmanMeas4 *meas)
// {
//   const CvMat *pred;
//   CvMat *measure = cvCreateMat( MEAS_SIZE,1, CV_32FC1);
  
//   pred = cvKalmanPredict(kalf,0);

//   if (meas == NULL) /* use the prediction */
//   {
// 	  cvmSet(measure,0,0, *(kalf->state_pre->data.fl));
// 	  cvmSet(measure,1,0, *(kalf->state_pre->data.fl+1));
// 	  cvmSet(measure,2,0, *(kalf->state_pre->data.fl+2));
// 	  cvmSet(measure,3,0, *(kalf->state_pre->data.fl+3));
//   }
//   else  /* use the measurement */
//   {
// 	  cvmSet(measure,0,0, meas->x);
// 	  cvmSet(measure,1,0, meas->y);
// 	  cvmSet(measure,2,0, meas->w);
// 	  cvmSet(measure,3,0, meas->h);
//   }

//   cvKalmanCorrect(kalf, measure);

//   cvReleaseMat( &measure );
// }
